#include "scpiparser.h"
#include "scpi_user.h"

#ifdef __cplusplus

  extern "C" {
  
#endif

struct scpi_parser_context ctx;
uint8_t iPins[NUMBEROFCHANNELS] = {22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53};
uint8_t bDigital[(NUMBEROFCHANNELS+7)/8];
char* token_list[MAX_TOKENS];

void scpi_user_init(void) {
   // static command tree in cmd_tree.c
}

/*
 * Respond to *IDN?
 */
scpi_error_t identify(struct scpi_parser_context* context, const char* token[]) {

  (void)context;  // dummy for compiler warning suppression
  (void)token;    // dummy for compiler warning suppression
  Serial.println(F("Relay control unit 1, SCPI Switch, v0.01,"));
  return SCPI_SUCCESS;
}


/*
 * Respond to *RST and *CLS
 */
scpi_error_t reset(struct scpi_parser_context* context, const char* token[]) {
  int i = 0;

  (void)context;  // dummy for compiler warning suppression
  (void)token;    // dummy for compiler warning suppression

  for (i = 0; i < NUMBEROFCHANNELS; i++) {
    digitalWrite(iPins[i], LOW);
  }
  memset(bDigital, 0, sizeof(bDigital));
  return SCPI_SUCCESS;
}


scpi_error_t get_digital(struct scpi_parser_context* context, const char* token[], int channel) {
  uint8_t bit, byte;
  
  (void)context;  // dummy for compiler warning suppression
  (void)token;    // dummy for compiler warning suppression

  // saving RAM by using the output registers to store the channel state
  bit = channel % 8;
  byte = channel / 8;
  if ( bDigital[byte] & (1<<bit) ) {
    Serial.println(F("ON"));
  } else {
    Serial.println(F("OFF"));   
  }
  return SCPI_SUCCESS;  
}


scpi_error_t set_digital(struct scpi_parser_context* context, const char* token[], int channel) {
  bool bSuccess = false;
  bool bSwitch = false;
  int i;
  uint8_t bit, byte;

  bit = channel % 8;
  byte = channel / 8;
  (void)context;  // dummy for warning suppression
//  Serial.print(F("Set digital ")); Serial.print(channel+1); Serial.print(' ');  // DEBUG

  // search last token
  for (i=0; token[i] != NULL; i++);
  i--;
 
 // Serial.print(token[i]); Serial.print(' ');   // DEBUG
  
  if (       strcmp_P(token[i], PSTR("0")) == 0
      || strcasecmp_P(token[i], PSTR("off")) == 0 ) {
        
      bSuccess = true;
  } else if (       strcmp_P(token[i], PSTR("1")) == 0 
             || strcasecmp_P(token[i], PSTR("on")) == 0 ){
      
      bSuccess = true;
      bSwitch = true;
  }

  if (bSuccess) {
    if (bSwitch) {      
//      Serial.println(F("HIGH"));    // DEBUG
      digitalWrite(iPins[channel], HIGH);
       bDigital[byte] |= (1<<bit);
    } else {      
//      Serial.println(F("LOW"));     // DEBUG
      digitalWrite(iPins[channel], LOW);
      bDigital[byte] &= ~(1<<bit);
    }
    return SCPI_SUCCESS;
  } else {
//    Serial.println(F("FAIL"));     // DEBUG
    return SCPI_COMMAND_NOT_FOUND;  
  }   
}

/**
 * get digital 1-32 status
 */
scpi_error_t get_digital_1(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 0);
}

scpi_error_t get_digital_2(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 1);
}

scpi_error_t get_digital_3(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 2);
}

scpi_error_t get_digital_4(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 3);
}

scpi_error_t get_digital_5(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 4);
}

scpi_error_t get_digital_6(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 5);
}

scpi_error_t get_digital_7(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 6);
}

scpi_error_t get_digital_8(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 7);
}

scpi_error_t get_digital_9(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 8);
}

scpi_error_t get_digital_10(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 9);
}

scpi_error_t get_digital_11(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 10);
}

scpi_error_t get_digital_12(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 11);
}

scpi_error_t get_digital_13(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 12);
}

scpi_error_t get_digital_14(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 13);
}

scpi_error_t get_digital_15(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 14);
}

scpi_error_t get_digital_16(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 15);
}

scpi_error_t get_digital_17(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 16);
}

scpi_error_t get_digital_18(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 17);
}

scpi_error_t get_digital_19(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 18);
}

scpi_error_t get_digital_20(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 19);
}

scpi_error_t get_digital_21(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 20);
}

scpi_error_t get_digital_22(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 21);
}

scpi_error_t get_digital_23(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 22);
}

scpi_error_t get_digital_24(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 23);
}

scpi_error_t get_digital_25(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 24);
}

scpi_error_t get_digital_26(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 25);
}

scpi_error_t get_digital_27(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 26);
}

scpi_error_t get_digital_28(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 27);
}

scpi_error_t get_digital_29(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 28);
}

scpi_error_t get_digital_30(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 29);
}

scpi_error_t get_digital_31(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 30);
}

scpi_error_t get_digital_32(struct scpi_parser_context* context, const char* token[]) {
  return get_digital(context, token, 31);
}

scpi_error_t set_digital_1(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 0);
}

scpi_error_t set_digital_2(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 1);
}

scpi_error_t set_digital_3(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 2);
}

scpi_error_t set_digital_4(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 3);
}

scpi_error_t set_digital_5(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 4);
}

scpi_error_t set_digital_6(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 5);
}

scpi_error_t set_digital_7(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 6);
}

scpi_error_t set_digital_8(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 7);
}

scpi_error_t set_digital_9(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 8);
}

scpi_error_t set_digital_10(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 9);
}

scpi_error_t set_digital_11(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 10);
}

scpi_error_t set_digital_12(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 11);
}

scpi_error_t set_digital_13(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 12);
}

scpi_error_t set_digital_14(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 13);
}

scpi_error_t set_digital_15(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 14);
}

scpi_error_t set_digital_16(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 15);
}

scpi_error_t set_digital_17(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 16);
}

scpi_error_t set_digital_18(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 17);
}

scpi_error_t set_digital_19(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 18);
}

scpi_error_t set_digital_20(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 19);
}

scpi_error_t set_digital_21(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 20);
}

scpi_error_t set_digital_22(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 21);
}

scpi_error_t set_digital_23(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 22);
}

scpi_error_t set_digital_24(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 23);
}

scpi_error_t set_digital_25(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 24);
}

scpi_error_t set_digital_26(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 25);
}

scpi_error_t set_digital_27(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 26);
}

scpi_error_t set_digital_28(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 27);
}

scpi_error_t set_digital_29(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 28);
}

scpi_error_t set_digital_30(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 29);
}

scpi_error_t set_digital_31(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 30);
}

scpi_error_t set_digital_32(struct scpi_parser_context* context, const char* token[]) {
  return set_digital(context, token, 31);
}

#ifdef __cplusplus
  }
#endif
